@@ -47,8 +47,7 @@ from phonopy.harmonic.force_constants import (
     rotational_invariance,
     cutoff_force_constants,
     set_tensor_symmetry)
-from phonopy.harmonic.dynamical_matrix import (DynamicalMatrix,
-                                               DynamicalMatrixNAC)
+from phonopy.harmonic.dynamical_matrix import get_dynamical_matrix
 from phonopy.phonon.band_structure import BandStructure
 from phonopy.phonon.thermal_properties import ThermalProperties
 from phonopy.phonon.mesh import Mesh, IterMesh
@@ -73,6 +72,7 @@ class Phonopy(object):
                  nac_params=None,
                  distance=None,
                  factor=VaspToTHz,
+                 frequency_scale_factor=None,
                  is_auto_displacements=None,
                  dynamical_matrix_decimals=None,
                  force_constants_decimals=None,
@@ -98,6 +98,7 @@ class Phonopy(object):
 
         self._symprec = symprec
         self._factor = factor
+        self._frequency_scale_factor = frequency_scale_factor
         self._is_symmetry = is_symmetry
         self._use_lapack_solver = use_lapack_solver
         self._log_level = log_level
@@ -348,7 +349,7 @@ class Phonopy(object):
             self._supercell)
         self.set_displacement_dataset(displacement_dataset)
 
-    def produce_force_constants(self,
+    def  produce_force_constants(self,
                                 forces=None,
                                 calculate_full_force_constants=True,
                                 computation_algorithm="svd"):
@@ -1238,7 +1239,8 @@ class Phonopy(object):
             self._dynamical_matrix,
             q_length=q_length,
             symmetry=self._primitive_symmetry,
-            frequency_factor_to_THz=self._factor)
+            frequency_factor_to_THz=self._factor,
+            log_level=self._log_level)
         return True
 
     def get_group_velocity(self):
@@ -1250,6 +1252,9 @@ class Phonopy(object):
         self._group_velocity.set_q_points([q_point])
         return self._group_velocity.get_group_velocity()[0]
 
+    def get_group_velocities_on_bands(self):
+        return self._band_structure.get_group_velocities()
+
     # Moment
     def set_moment(self,
                    order=1,
@@ -1311,22 +1316,32 @@ class Phonopy(object):
             print("Warning: Atomic masses are not correctly set.")
             return False
         else:
-            if self._nac_params is None:
-                self._dynamical_matrix = DynamicalMatrix(
-                    self._supercell,
-                    self._primitive,
-                    self._force_constants,
-                    decimals=self._dynamical_matrix_decimals,
-                    symprec=self._symprec)
-            else:
-                self._dynamical_matrix = DynamicalMatrixNAC(
-                    self._supercell,
-                    self._primitive,
-                    self._force_constants,
-                    nac_params=self._nac_params,
-                    decimals=self._dynamical_matrix_decimals,
-                    symprec=self._symprec,
-                    log_level=self._log_level)
+            self._dynamical_matrix = get_dynamical_matrix(
+                self._force_constants,
+                self._supercell,
+                self._primitive,
+                self._nac_params,
+                self._frequency_scale_factor,
+                self._dynamical_matrix_decimals,
+                symprec=self._symprec,
+                log_level=self._log_level)
+
+            # if self._nac_params is None:
+            #     self._dynamical_matrix = DynamicalMatrix(
+            #         self._supercell,
+            #         self._primitive,
+            #         self._force_constants,
+            #         decimals=self._dynamical_matrix_decimals,
+            #         symprec=self._symprec)
+            # else:
+            #     self._dynamical_matrix = DynamicalMatrixNAC(
+            #         self._supercell,
+            #         self._primitive,
+            #         self._force_constants,
+            #         nac_params=self._nac_params,
+            #         decimals=self._dynamical_matrix_decimals,
+            #         symprec=self._symprec,
+            #         log_level=self._log_level)
             return True
 
     def _search_symmetry(self):
